@@ -1,21 +1,7 @@
 /**
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "lanlinkprovider.h"
@@ -37,19 +23,25 @@
 #include <QNetworkConfigurationManager>
 #include <QSslCipher>
 #include <QSslConfiguration>
+#include <QSslKey>
+#include <QStringList>
 
 #include "daemon.h"
 #include "landevicelink.h"
 #include "lanpairinghandler.h"
 #include "kdeconnectconfig.h"
+#include "qtcompat_p.h"
 
 #define MIN_VERSION_WITH_SSL_SUPPORT 6
 
+static const int MAX_UNPAIRED_CONNECTIONS = 42;
+static const int MAX_REMEMBERED_IDENTITY_PACKETS = 42;
+
 LanLinkProvider::LanLinkProvider(
         bool testMode,
         quint16 udpBroadcastPort,
         quint16 udpListenPort
-    )
+        )
     : m_server(new Server(this))
     , m_udpSocket(this)
     , m_tcpPort(0)
@@ -98,10 +90,10 @@ void LanLinkProvider::onStart()
         // Refer to https://doc.qt.io/qt-5/qabstractsocket.html#SocketError-enum to decode socket error number
         QString errorMessage = QString::fromLatin1(QMetaEnum::fromType<QAbstractSocket::SocketError>().valueToKey(sockErr));
         qCritical(KDECONNECT_CORE)
-            << QLatin1String("Failed to bind UDP socket on port")
-            << m_udpListenPort
-            << QLatin1String("with error")
-            << errorMessage;
+                << QLatin1String("Failed to bind UDP socket on port")
+                << m_udpListenPort
+                << QLatin1String("with error")
+                << errorMessage;
     }
     Q_ASSERT(success);
 
@@ -129,7 +121,7 @@ void LanLinkProvider::onStop()
 void LanLinkProvider::onNetworkChange()
 {
     if (m_combineBroadcastsTimer.isActive()) {
-        qCDebug(KDECONNECT_CORE()) << "Preventing duplicate broadcasts";
+        qCDebug(KDECONNECT_CORE) << "Preventing duplicate broadcasts";
         return;
     }
     m_combineBroadcastsTimer.start();
@@ -138,7 +130,6 @@ void LanLinkProvider::onNetworkChange()
 //I'm in a new network, let's be polite and introduce myself
 void LanLinkProvider::broadcastToNetwork()
 {
-
     if (!m_server->isListening()) {
         //Not started
         return;
@@ -148,11 +139,16 @@ void LanLinkProvider::broadcastToNetwork()
 
     qCDebug(KDECONNECT_CORE()) << "Broadcasting identity packet";
 
-    QHostAddress destAddress = m_testMode? QHostAddress::LocalHost : QHostAddress(QStringLiteral("255.255.255.255"));
+    QList<QHostAddress> destinations = getBroadcastAddresses();
 
-    NetworkPacket np(QLatin1String(""));
+    NetworkPacket np;
     NetworkPacket::createIdentityPacket(&np);
     np.set(QStringLiteral("tcpPort"), m_tcpPort);
+#ifdef Q_OS_MAC
+    //On macOS, remove capacitilities to avoid incomplete transmission of too large UDP packet
+    np.set(QStringLiteral("incomingCapabilities"), QStringList());
+    np.set(QStringLiteral("outgoingCapabilities"), QStringList());
+#endif
 
 #ifdef Q_OS_WIN
     //On Windows we need to broadcast from every local IP address to reach all networks
@@ -160,22 +156,54 @@ void LanLinkProvider::broadcastToNetwork()
     sendSocket.setProxy(QNetworkProxy::NoProxy);
     for (const QNetworkInterface& iface : QNetworkInterface::allInterfaces()) {
         if ( (iface.flags() & QNetworkInterface::IsUp)
-          && (iface.flags() & QNetworkInterface::IsRunning)
-          && (iface.flags() & QNetworkInterface::CanBroadcast)) {
+             && (iface.flags() & QNetworkInterface::IsRunning)
+             && (iface.flags() & QNetworkInterface::CanBroadcast)) {
             for (const QNetworkAddressEntry& ifaceAddress : iface.addressEntries()) {
                 QHostAddress sourceAddress = ifaceAddress.ip();
                 if (sourceAddress.protocol() == QAbstractSocket::IPv4Protocol && sourceAddress != QHostAddress::LocalHost) {
                     qCDebug(KDECONNECT_CORE()) << "Broadcasting as" << sourceAddress;
-                    sendSocket.writeDatagram(np.serialize(), destAddress, m_udpBroadcastPort);
+                    sendBroadcasts(sendSocket, np, destinations);
                     sendSocket.close();
                 }
             }
         }
     }
 #else
-    m_udpSocket.writeDatagram(np.serialize(), destAddress, m_udpBroadcastPort);
+    sendBroadcasts(m_udpSocket, np, destinations);
 #endif
+}
 
+QList<QHostAddress> LanLinkProvider::getBroadcastAddresses()
+{
+    const QStringList customDevices = KdeConnectConfig::instance().customDevices();
+
+    QList<QHostAddress> destinations;
+    destinations.reserve(customDevices.length() + 1);
+
+    // Default broadcast address
+    destinations.append(m_testMode ? QHostAddress::LocalHost : QHostAddress::Broadcast);
+
+    // Custom device addresses
+    for (auto& customDevice : customDevices) {
+        QHostAddress address(customDevice);
+        if (address.isNull()) {
+            qCWarning(KDECONNECT_CORE) << "Invalid custom device address" << customDevice;
+        } else {
+            destinations.append(address);
+        }
+    }
+
+    return destinations;
+}
+
+void LanLinkProvider::sendBroadcasts(
+        QUdpSocket& socket, const NetworkPacket& np, const QList<QHostAddress>& addresses)
+{
+    const QByteArray payload = np.serialize();
+
+    for (auto& address : addresses) {
+        socket.writeDatagram(payload, address, m_udpBroadcastPort);
+    }
 }
 
 //I'm the existing device, a new device is kindly introducing itself.
@@ -219,15 +247,30 @@ void LanLinkProvider::udpBroadcastReceived()
         }
 
         int tcpPort = receivedPacket->get<int>(QStringLiteral("tcpPort"));
+        if (tcpPort < MIN_TCP_PORT || tcpPort > MAX_TCP_PORT) {
+            qCDebug(KDECONNECT_CORE) << "TCP port outside of kdeconnect's range";
+            delete receivedPacket;
+            continue;
+        }
 
         //qCDebug(KDECONNECT_CORE) << "Received Udp identity packet from" << sender << " asking for a tcp connection on port " << tcpPort;
 
+        if (m_receivedIdentityPackets.size() > MAX_REMEMBERED_IDENTITY_PACKETS) {
+            qCWarning(KDECONNECT_CORE) << "Too many remembered identities, ignoring" << receivedPacket->get<QString>(QStringLiteral("deviceId")) << "received via UDP";
+            delete receivedPacket;
+            continue;
+        }
+
         QSslSocket* socket = new QSslSocket(this);
         socket->setProxy(QNetworkProxy::NoProxy);
         m_receivedIdentityPackets[socket].np = receivedPacket;
         m_receivedIdentityPackets[socket].sender = sender;
         connect(socket, &QAbstractSocket::connected, this, &LanLinkProvider::tcpSocketConnected);
+#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
         connect(socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error), this, &LanLinkProvider::connectError);
+#else
+        connect(socket, &QAbstractSocket::errorOccurred, this, &LanLinkProvider::connectError);
+#endif
         socket->connectToHost(sender, tcpPort);
     }
 }
@@ -247,7 +290,7 @@ void LanLinkProvider::connectError(QAbstractSocket::SocketError socketError)
     //The socket we created didn't work, and we didn't manage
     //to create a LanDeviceLink from it, deleting everything.
     delete m_receivedIdentityPackets.take(socket).np;
-    delete socket;
+    socket->deleteLater();
 }
 
 //We received a UDP packet and answered by connecting to them by TCP. This gets called on a successful connection.
@@ -257,7 +300,11 @@ void LanLinkProvider::tcpSocketConnected()
 
     if (!socket) return;
     // TODO Delete me?
+#if QT_VERSION < QT_VERSION_CHECK(5,15,0)
     disconnect(socket, QOverload<QAbstractSocket::SocketError>::of(&QAbstractSocket::error), this, &LanLinkProvider::connectError);
+#else
+    disconnect(socket, &QAbstractSocket::errorOccurred, this, &LanLinkProvider::connectError);
+#endif
 
     configureSocket(socket);
 
@@ -288,9 +335,7 @@ void LanLinkProvider::tcpSocketConnected()
 
             connect(socket, &QSslSocket::encrypted, this, &LanLinkProvider::encrypted);
 
-            if (isDeviceTrusted) {
-                connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
-            }
+            connect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
 
             socket->startServerEncryption();
 
@@ -317,14 +362,18 @@ void LanLinkProvider::encrypted()
 
     QSslSocket* socket = qobject_cast<QSslSocket*>(sender());
     if (!socket) return;
-    // TODO delete me?
-    disconnect(socket, QOverload<const QList<QSslError> &>::of(&QSslSocket::sslErrors), this, &LanLinkProvider::sslErrors);
 
     Q_ASSERT(socket->mode() != QSslSocket::UnencryptedMode);
     LanDeviceLink::ConnectionStarted connectionOrigin = (socket->mode() == QSslSocket::SslClientMode)? LanDeviceLink::Locally : LanDeviceLink::Remotely;
 
     NetworkPacket* receivedPacket = m_receivedIdentityPackets[socket].np;
-    const QString& deviceId = receivedPacket->get<QString>(QStringLiteral("deviceId"));
+    const QString& deviceId = socket->peerCertificate().subjectDisplayName();
+
+    if (m_links.contains(deviceId) && m_links[deviceId]->certificate() != socket->peerCertificate()) {
+        socket->disconnectFromHost();
+        qCWarning(KDECONNECT_CORE) << "Got connection for the same deviceId but certificates don't match. Ignoring " << deviceId;
+        return;
+    }
 
     addLink(deviceId, socket, receivedPacket, connectionOrigin);
 
@@ -337,14 +386,20 @@ void LanLinkProvider::sslErrors(const QList<QSslError>& errors)
     QSslSocket* socket = qobject_cast<QSslSocket*>(sender());
     if (!socket) return;
 
-    qCDebug(KDECONNECT_CORE) << "Failing due to " << errors;
-    Device* device = Daemon::instance()->getDevice(socket->peerVerifyName());
-    if (device) {
-        device->unpair();
+    bool fatal = false;
+    for (const QSslError& error : errors) {
+        if (error.error() != QSslError::SelfSignedCertificate) {
+            qCCritical(KDECONNECT_CORE) << "Disconnecting due to fatal SSL Error: " << error;
+            fatal = true;
+        } else {
+            qCDebug(KDECONNECT_CORE) << "Ignoring self-signed cert error";
+        }
     }
 
-    delete m_receivedIdentityPackets.take(socket).np;
-    // Socket disconnects itself on ssl error and will be deleted by deleteLater slot, no need to delete manually
+    if (fatal) {
+        socket->disconnectFromHost();
+        delete m_receivedIdentityPackets.take(socket).np;
+    }
 }
 
 //I'm the new device and this is the answer to my UDP identity packet (no data received yet). They are connecting to us through TCP, and they should send an identity.
@@ -363,6 +418,16 @@ void LanLinkProvider::newConnection()
         connect(socket, &QIODevice::readyRead,
                 this, &LanLinkProvider::dataReceived);
 
+        QTimer* timer = new QTimer(socket);
+        timer->setSingleShot(true);
+        timer->setInterval(1000);
+        connect(socket, &QSslSocket::encrypted,
+                timer, &QObject::deleteLater);
+        connect(timer, &QTimer::timeout, socket, [socket] {
+            qCWarning(KDECONNECT_CORE) << "LanLinkProvider/newConnection: Host timed out without sending any identity." << socket->peerAddress();
+            socket->disconnectFromHost();
+        });
+        timer->start();
     }
 }
 
@@ -370,7 +435,20 @@ void LanLinkProvider::newConnection()
 void LanLinkProvider::dataReceived()
 {
     QSslSocket* socket = qobject_cast<QSslSocket*>(sender());
+    //the size here is arbitrary and is now at 8192 bytes. It needs to be considerably long as it includes the capabilities but there needs to be a limit
+    //Tested between my systems and I get around 2000 per identity package.
+    if (socket->bytesAvailable() > 8192) {
+        qCWarning(KDECONNECT_CORE) << "LanLinkProvider/newConnection: Suspiciously long identity package received. Closing connection." << socket->peerAddress() << socket->bytesAvailable();
+        socket->disconnectFromHost();
+        return;
+    }
+
+#if QT_VERSION < QT_VERSION_CHECK(5,7,0)
+    if (!socket->canReadLine())
+        return;
+#else
     socket->startTransaction();
+#endif
 
     const QByteArray data = socket->readLine();
 
@@ -379,12 +457,19 @@ void LanLinkProvider::dataReceived()
     NetworkPacket* np = new NetworkPacket(QLatin1String(""));
     bool success = NetworkPacket::unserialize(data, np);
 
+#if QT_VERSION < QT_VERSION_CHECK(5,7,0)
+    if (!success) {
+        delete np;
+        return;
+    }
+#else
     if (!success) {
         delete np;
         socket->rollbackTransaction();
         return;
     }
     socket->commitTransaction();
+#endif
 
     if (np->type() != PACKET_TYPE_IDENTITY) {
         qCWarning(KDECONNECT_CORE) << "LanLinkProvider/newConnection: Expected identity, received " << np->type();
@@ -392,6 +477,12 @@ void LanLinkProvider::dataReceived()
         return;
     }
 
+    if (m_receivedIdentityPackets.size() > MAX_REMEMBERED_IDENTITY_PACKETS) {
+        qCWarning(KDECONNECT_CORE) << "Too many remembered identities, ignoring" << np->get<QString>(QStringLiteral("deviceId")) << "received via TCP";
+        delete np;
+        return;
+    }
+
     // Needed in "encrypted" if ssl is used, similar to "tcpSocketConnected"
     m_receivedIdentityPackets[socket].np = np;
 
@@ -451,19 +542,26 @@ void LanLinkProvider::configureSslSocket(QSslSocket* socket, const QString& devi
     // Configure for ssl
     QSslConfiguration sslConfig;
     sslConfig.setCiphers(socketCiphers);
+    sslConfig.setLocalCertificate(KdeConnectConfig::instance().certificate());
 
-    socket->setSslConfiguration(sslConfig);
-    socket->setLocalCertificate(KdeConnectConfig::instance().certificate());
-    socket->setPrivateKey(KdeConnectConfig::instance().privateKeyPath());
-    socket->setPeerVerifyName(deviceId);
+    QFile privateKeyFile(KdeConnectConfig::instance().privateKeyPath());
+    QSslKey privateKey;
+    if (privateKeyFile.open(QIODevice::ReadOnly)) {
+        privateKey = QSslKey(privateKeyFile.readAll(), QSsl::Rsa);
+    }
+    privateKeyFile.close();
+    sslConfig.setPrivateKey(privateKey);
 
     if (isDeviceTrusted) {
         QString certString = KdeConnectConfig::instance().getDeviceProperty(deviceId, QStringLiteral("certificate"), QString());
-        socket->addCaCertificate(QSslCertificate(certString.toLatin1()));
-        socket->setPeerVerifyMode(QSslSocket::VerifyPeer);
+        sslConfig.setCaCertificates({QSslCertificate(certString.toLatin1())});
+        sslConfig.setPeerVerifyMode(QSslSocket::VerifyPeer);
     } else {
-        socket->setPeerVerifyMode(QSslSocket::QueryPeer);
+        sslConfig.setPeerVerifyMode(QSslSocket::QueryPeer);
     }
+    socket->setSslConfiguration(sslConfig);
+    socket->setPeerVerifyName(deviceId);
+
 
     //Usually SSL errors are only bad for trusted devices. Uncomment this section to log errors in any case, for debugging.
     //QObject::connect(socket, static_cast<void (QSslSocket::*)(const QList<QSslError>&)>(&QSslSocket::sslErrors), [](const QList<QSslError>& errors)
@@ -480,23 +578,23 @@ void LanLinkProvider::configureSocket(QSslSocket* socket) {
 
     socket->setSocketOption(QAbstractSocket::KeepAliveOption, QVariant(1));
 
-    #ifdef TCP_KEEPIDLE
-        // time to start sending keepalive packets (seconds)
-        int maxIdle = 10;
-        setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPIDLE, &maxIdle, sizeof(maxIdle));
-    #endif
+#ifdef TCP_KEEPIDLE
+    // time to start sending keepalive packets (seconds)
+    int maxIdle = 10;
+    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPIDLE, &maxIdle, sizeof(maxIdle));
+#endif
 
-    #ifdef TCP_KEEPINTVL
-        // interval between keepalive packets after the initial period (seconds)
-        int interval = 5;
-        setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPINTVL, &interval, sizeof(interval));
-    #endif
+#ifdef TCP_KEEPINTVL
+    // interval between keepalive packets after the initial period (seconds)
+    int interval = 5;
+    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPINTVL, &interval, sizeof(interval));
+#endif
 
-    #ifdef TCP_KEEPCNT
-        // number of missed keepalive packets before disconnecting
-        int count = 3;
-        setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPCNT, &count, sizeof(count));
-    #endif
+#ifdef TCP_KEEPCNT
+    // number of missed keepalive packets before disconnecting
+    int count = 3;
+    setsockopt(socket->socketDescriptor(), IPPROTO_TCP, TCP_KEEPCNT, &count, sizeof(count));
+#endif
 
 }
 
@@ -514,6 +612,15 @@ void LanLinkProvider::addLink(const QString& deviceId, QSslSocket* socket, Netwo
         deviceLink->reset(socket, connectionOrigin);
     } else {
         deviceLink = new LanDeviceLink(deviceId, this, socket, connectionOrigin);
+        // Socket disconnection will now be handled by LanDeviceLink
+        disconnect(socket, &QAbstractSocket::disconnected, socket, &QObject::deleteLater);
+        bool isDeviceTrusted = KdeConnectConfig::instance().trustedDevices().contains(deviceId);
+        if (!isDeviceTrusted && m_links.size() > MAX_UNPAIRED_CONNECTIONS) {
+            qCWarning(KDECONNECT_CORE) << "Too many unpaired devices to remember them all. Ignoring " << deviceId;
+            socket->disconnectFromHost();
+            socket->deleteLater();
+            return;
+        }
         connect(deviceLink, &QObject::destroyed, this, &LanLinkProvider::deviceLinkDestroyed);
         m_links[deviceId] = deviceLink;
         if (m_pairingHandlers.contains(deviceId)) {
