@@ -1,21 +1,7 @@
 /**
- * Copyright 2013 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "networkpacket.h"
@@ -71,32 +57,17 @@ void NetworkPacket::createIdentityPacket(NetworkPacket* np)
     //qCDebug(KDECONNECT_CORE) << "createIdentityPacket" << np->serialize();
 }
 
-template<class T>
-QVariantMap qobject2qvariant(const T* object)
-{
-    QVariantMap map;
-    auto metaObject = T::staticMetaObject;
-    for(int i = metaObject.propertyOffset(); i < metaObject.propertyCount(); ++i) {
-        QMetaProperty prop = metaObject.property(i);
-        map.insert(QString::fromLatin1(prop.name()), prop.readOnGadget(object));
-    }
-
-    return map;
-}
-
 QByteArray NetworkPacket::serialize() const
 {
     //Object -> QVariant
-    //QVariantMap variant;
-    //variant["id"] = mId;
-    //variant["type"] = mType;
-    //variant["body"] = mBody;
-    QVariantMap variant = qobject2qvariant(this);
+    QVariantMap variant;
+    variant.insert(QStringLiteral("id"), m_id);
+    variant.insert(QStringLiteral("type"), m_type);
+    variant.insert(QStringLiteral("body"), m_body);
 
     if (hasPayload()) {
-        //qCDebug(KDECONNECT_CORE) << "Serializing payloadTransferInfo";
-        variant[QStringLiteral("payloadSize")] = payloadSize();
-        variant[QStringLiteral("payloadTransferInfo")] = m_payloadTransferInfo;
+        variant.insert(QStringLiteral("payloadSize"), m_payloadSize);
+        variant.insert(QStringLiteral("payloadTransferInfo"), m_payloadTransferInfo);
     }
 
     //QVariant -> json
@@ -147,7 +118,7 @@ bool NetworkPacket::unserialize(const QByteArray& a, NetworkPacket* np)
     qvariant2qobject(variant, np);
 
     if (np->m_payloadSize == -1) {
-        np->m_payloadSize = np->get<int>(QStringLiteral("size"), -1);
+        np->m_payloadSize = np->get<qint64>(QStringLiteral("size"), -1);
     }
     np->m_payloadTransferInfo = variant[QStringLiteral("payloadTransferInfo")].toMap(); //Will return an empty qvariantmap if was not present, which is ok
 
