@@ -38,7 +38,7 @@ public:
         : Daemon(parent)
         , m_nam(nullptr)
     {
-        qApp->setWindowIcon(QIcon(QStandardPaths::locate(QStandardPaths::AppLocalDataLocation, QStringLiteral("icons/hicolor/scalable/apps/kdeconnect.svg"))));
+        qApp->setWindowIcon(QIcon(QStringLiteral(":/icons/kdeconnect/kdeconnect.svg")));
     }
 
     void askPairingConfirmation(Device* device) override
@@ -48,7 +48,7 @@ public:
         notification->setIconName(QStringLiteral("dialog-information"));
         notification->setComponentName(QStringLiteral("kdeconnect"));
         notification->setTitle(QStringLiteral("KDE Connect"));
-        notification->setText(i18n("Pairing request from %1\n<br/>Key: %2...", device->name().toHtmlEscaped(), QString::fromUtf8(device->verificationKey().left(8))));
+        notification->setText(i18n("Pairing request from %1\nKey: %2...", device->name().toHtmlEscaped(), QString::fromUtf8(device->verificationKey().left(8))));
         notification->setDefaultAction(i18n("Open"));
         notification->setActions(QStringList() << i18n("Accept") << i18n("Reject") << i18n("View key"));
         connect(notification, &KNotification::action1Activated, device, &Device::acceptPairing);
@@ -163,6 +163,11 @@ int main(int argc, char* argv[])
 
     DesktopDaemon daemon;
 
+#ifdef Q_OS_WIN
+    // make sure indicator shows up in the tray whenever daemon is spawned
+    QProcess::startDetached(QStringLiteral("kdeconnect-indicator.exe"));
+#endif
+
     // kdeconnectd is autostarted, so disable session management to speed up startup
     auto disableSessionManagement = [](QSessionManager &sm) {
         sm.setRestartHint(QSessionManager::RestartNever);
