@@ -1,77 +1,51 @@
 /*
  * SPDX-FileCopyrightText: 2016 Aleix Pol Gonzalez <aleixpol@kde.org>
+ * SPDX-FileCopyrightText: 2020 Piyush Aggarwal <piyushaggarwal002@gmail.com>
  *
  * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "deviceindicator.h"
 #include <QFileDialog>
+#include <QStandardPaths>
+#include <QDateTime>
 #include <KLocalizedString>
 
 #include "interfaces/dbusinterfaces.h"
 
 #include <dbushelper.h>
-
-class BatteryAction : public QAction
-{
-Q_OBJECT
-public:
-    BatteryAction(DeviceDbusInterface* device)
-        : QAction(nullptr)
-        , m_batteryIface(device->id())
-    {
-        setCharge(m_batteryIface.charge());
-        setCharging(m_batteryIface.isCharging());
-
-        connect(&m_batteryIface, &BatteryDbusInterface::refreshedProxy, this, [this]{
-            setCharge(m_batteryIface.charge());
-            setCharging(m_batteryIface.isCharging());
-        });
-
-        setIcon(QIcon::fromTheme(QStringLiteral("battery")));
-
-        update();
-    }
-
-    void update() {
-        if (m_charge < 0)
-            setText(i18n("No Battery"));
-        else if (m_charging)
-            setText(i18n("Battery: %1% (Charging)", m_charge));
-        else
-            setText(i18n("Battery: %1%", m_charge));
-    }
-
-private Q_SLOTS:
-    void setCharge(int charge) { m_charge = charge; update(); }
-    void setCharging(bool charging) { m_charging = charging; update(); }
-
-private:
-    BatteryDbusInterface m_batteryIface;
-    int m_charge = -1;
-    bool m_charging = false;
-};
-
+#include <dbushelpers.h>
+#include <systray_actions.h>
 
 DeviceIndicator::DeviceIndicator(DeviceDbusInterface* device)
     : QMenu(device->name(), nullptr)
     , m_device(device)
     , m_remoteCommandsInterface(new RemoteCommandsDbusInterface(m_device->id()))
 {
-#ifdef Q_OS_WIN
-    setIcon(QIcon(QStandardPaths::locate(QStandardPaths::AppDataLocation, QStringLiteral("icons/hicolor/scalable/status/") + device->iconName() + QStringLiteral(".svg"))));
-#else
     setIcon(QIcon::fromTheme(device->iconName()));
-#endif
 
     connect(device, SIGNAL(nameChanged(QString)), this, SLOT(setText(QString)));
 
+    // Battery status
     auto battery = new BatteryAction(device);
     addAction(battery);
     setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_battery")),
-                     [battery](bool available) { battery->setVisible(available);  }
-                     , this);
+                    [battery](bool available) {
+                        battery->setVisible(available); 
+                        battery->setDisabled(available); 
+                    }, this);
 
+    auto connectivity = new ConnectivityAction(device);
+    addAction(connectivity);
+    setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_connectivity_report")),
+                    [connectivity](bool available) {
+                        connectivity->setVisible(available); 
+                        connectivity->setDisabled(available); 
+                    }, this);
+
+    this->addSeparator();
+
+    // Browse device filesystem
     auto browse = addAction(QIcon::fromTheme(QStringLiteral("document-open-folder")), i18n("Browse device"));
     connect(browse, &QAction::triggered, device, [device](){
         SftpDbusInterface* sftpIface = new SftpDbusInterface(device->id(), device);
@@ -80,6 +54,7 @@ DeviceIndicator::DeviceIndicator(DeviceDbusInterface* device)
     });
     setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_sftp")), [browse](bool available) { browse->setVisible(available); }, this);
 
+    // Find device
     auto findDevice = addAction(QIcon::fromTheme(QStringLiteral("irc-voice")), i18n("Ring device"));
     connect(findDevice, &QAction::triggered, device, [device](){
         FindMyPhoneDeviceDbusInterface* iface = new FindMyPhoneDeviceDbusInterface(device->id(), device);
@@ -88,20 +63,26 @@ DeviceIndicator::DeviceIndicator(DeviceDbusInterface* device)
     });
     setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_findmyphone")), [findDevice](bool available) { findDevice->setVisible(available); }, this);
 
-    auto sendFile = addAction(QIcon::fromTheme(QStringLiteral("document-share")), i18n("Send file"));
-    connect(sendFile, &QAction::triggered, device, [device, this](){
-        const QUrl url = QFileDialog::getOpenFileUrl(parentWidget(), i18n("Select file to send to '%1'", device->name()), QUrl::fromLocalFile(QDir::homePath()));
-        if (url.isEmpty())
-            return;
-
-        QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device->id() + QStringLiteral("/share"), QStringLiteral("org.kde.kdeconnect.device.share"), QStringLiteral("shareUrl"));
-        msg.setArguments(QVariantList() << url.toString());
-        DBusHelper::sessionBus().call(msg);
+    // Get a photo
+    auto getPhoto = addAction(QIcon::fromTheme(QStringLiteral("camera-photo")), i18n("Get a photo"));
+    connect(getPhoto, &QAction::triggered, this, [device](){
+        QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kdeconnect"), QStringLiteral("/modules/kdeconnect/devices/") + device->id() + QStringLiteral("/photo"), QStringLiteral("org.kde.kdeconnect.device.photo"), QStringLiteral("requestPhoto"));
+        msg.setArguments({QStandardPaths::standardLocations(QStandardPaths::DownloadLocation).first() + QDateTime::currentDateTime().toString(QStringLiteral("/dd-MM-yy_hh-mm-ss.png"))});
+        blockOnReply(DBusHelper::sessionBus().asyncCall(msg));
     });
+    setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_photo")), [getPhoto](bool available) { getPhoto->setVisible(available); }, this);
 
-    setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_share")), [sendFile](bool available) { sendFile->setVisible(available); }, this);
+    // Send file
+    const QString kdeconnectHandlerExecutable = QStandardPaths::findExecutable(QStringLiteral("kdeconnect-handler"), { QCoreApplication::applicationDirPath() });
+    if (!kdeconnectHandlerExecutable.isEmpty()) {
+        auto handlerApp = addAction(QIcon::fromTheme(QStringLiteral("document-share")), i18n("Send a file/URL"));
+        QObject::connect(handlerApp, &QAction::triggered, device, [device, kdeconnectHandlerExecutable] () {
+            QProcess::startDetached(kdeconnectHandlerExecutable, { QStringLiteral("--device"), device->id() });
+        });
+        handlerApp->setVisible(true);
+    }
 
-    // Search current application path
+    // SMS Messages
     const QString kdeconnectsmsExecutable = QStandardPaths::findExecutable(QStringLiteral("kdeconnect-sms"), { QCoreApplication::applicationDirPath() });
     if (!kdeconnectsmsExecutable.isEmpty()) {
         auto smsapp = addAction(QIcon::fromTheme(QStringLiteral("message-new")), i18n("SMS Messages..."));
@@ -111,6 +92,7 @@ DeviceIndicator::DeviceIndicator(DeviceDbusInterface* device)
         setWhenAvailable(device->hasPlugin(QStringLiteral("kdeconnect_sms")), [smsapp](bool available) { smsapp->setVisible(available); }, this);
     }
 
+    // Run command
     QMenu* remoteCommandsMenu = new QMenu(i18n("Run command"), this);
     QAction* menuAction = remoteCommandsMenu->menuAction();
     QAction* addCommandAction = remoteCommandsMenu->addAction(QIcon::fromTheme(QStringLiteral("list-add")), i18n("Add commands"));
@@ -135,5 +117,3 @@ DeviceIndicator::DeviceIndicator(DeviceDbusInterface* device)
         }
     }, this);
 }
-
-#include "deviceindicator.moc"
