@@ -11,10 +11,11 @@
 #include <KAboutData>
 #include <KLocalizedString>
 #include <kcmutils_version.h>
+#include <KColorSchemeManager>
 
 #include "ui_kcm.h"
-#include "interfaces/dbusinterfaces.h"
-#include "interfaces/devicesmodel.h"
+#include "dbusinterfaces.h"
+#include "devicesmodel.h"
 #include "devicessortproxymodel.h"
 #include "kdeconnect-version.h"
 
@@ -39,8 +40,15 @@ KdeConnectKcm::KdeConnectKcm(QWidget* parent, const QVariantList& args)
                                        QStringLiteral("https://community.kde.org/KDEConnect")
     );
     about->addAuthor(i18n("Albert Vaca Cintora"));
+    about->setProgramLogo(QIcon(QStringLiteral(":/icons/kdeconnect/kdeconnect.svg")));
+
     setAboutData(about);
 
+#ifdef Q_OS_WIN
+    KColorSchemeManager manager;
+    QApplication::setStyle(QStringLiteral("breeze"));
+#endif
+
     kcmUi->setupUi(this);
 
     sortProxyModel = new DevicesSortProxyModel(devicesModel);
