@@ -24,7 +24,7 @@ class MprisRemotePlayerMediaPlayer2 : public QDBusAbstractAdaptor {
 
 public:
     explicit MprisRemotePlayerMediaPlayer2(MprisRemotePlayer *parent, const MprisRemotePlugin *plugin);
-    ~MprisRemotePlayerMediaPlayer2();
+    ~MprisRemotePlayerMediaPlayer2() override;
 
 public Q_SLOTS:
     void Raise();
