@@ -1,21 +1,7 @@
 /**
- * Copyright 2015 Albert Vaca <albertvaka@gmail.com>
+ * SPDX-FileCopyrightText: 2015 Albert Vaca <albertvaka@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "sendreplydialog.h"
@@ -24,6 +10,8 @@
 #include <QTextEdit>
 #include <QLineEdit>
 #include <QBoxLayout>
+#include <QStandardPaths>
+#include <QKeyEvent>
 
 #include <KLocalizedString>
 
@@ -40,10 +28,14 @@ SendReplyDialog::SendReplyDialog(const QString& originalMessage, const QString&
     auto button = m_ui->buttonBox->button(QDialogButtonBox::Ok);
     button->setText(i18n("Send"));
 
+    auto textEdit = m_ui->replyEdit;
+    connect(textEdit, &SendReplyTextEdit::send, this, &SendReplyDialog::sendButtonClicked);
+
     connect(this, &QDialog::accepted, this, &SendReplyDialog::sendButtonClicked);
     setWindowTitle(topicName);
     setWindowIcon(QIcon::fromTheme(QStringLiteral("kdeconnect")));
     setAttribute(Qt::WA_DeleteOnClose);
+    m_ui->replyEdit->setFocus();
 }
 
 SendReplyDialog::~SendReplyDialog() = default;
@@ -58,3 +50,22 @@ QSize SendReplyDialog::sizeHint() const
 {
     return QSize(512, 64);
 }
+
+SendReplyTextEdit::SendReplyTextEdit(QWidget *parent)
+    : QTextEdit(parent)
+{
+}
+
+void SendReplyTextEdit::keyPressEvent(QKeyEvent* event)
+{
+    // Send reply on enter, except when shift + enter is pressed, then insert newline
+    const int key = event->key();
+    if (key == Qt::Key_Return || key == Qt::Key_Enter) {
+        if ((key == Qt::Key_Enter && (event->modifiers() == Qt::KeypadModifier)) || !event->modifiers()) {
+            Q_EMIT send();
+            event->accept();
+            return;
+        }
+    }
+    QTextEdit::keyPressEvent(event);
+}
