@@ -1,21 +1,7 @@
 /**
- * Copyright 2015 Holger Kaelberer <holger.k@elberer.de>
+ * SPDX-FileCopyrightText: 2015 Holger Kaelberer <holger.k@elberer.de>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "sendnotifications_config.h"
@@ -88,13 +74,13 @@ void SendNotificationsConfig::loadApplications()
 void SendNotificationsConfig::load()
 {
     KCModule::load();
-    bool persistent = config()->get(QStringLiteral("generalPersistent"), false);
+    bool persistent = config()->getBool(QStringLiteral("generalPersistent"), false);
     m_ui->check_persistent->setChecked(persistent);
-    bool body = config()->get(QStringLiteral("generalIncludeBody"), true);
+    bool body = config()->getBool(QStringLiteral("generalIncludeBody"), true);
     m_ui->check_body->setChecked(body);
-    bool icons = config()->get(QStringLiteral("generalSynchronizeIcons"), true);
+    bool icons = config()->getBool(QStringLiteral("generalSynchronizeIcons"), true);
     m_ui->check_icons->setChecked(icons);
-    int urgency = config()->get(QStringLiteral("generalUrgency"), 0);
+    int urgency = config()->getInt(QStringLiteral("generalUrgency"), 0);
     m_ui->spin_urgency->setValue(urgency);
 
     loadApplications();
@@ -109,8 +95,9 @@ void SendNotificationsConfig::save()
     config()->set(QStringLiteral("generalUrgency"), m_ui->spin_urgency->value());
 
     QVariantList list;
-    list.reserve(appModel->apps().size());
-    for (const auto& a: appModel->apps()) {
+    const auto apps = appModel->apps();
+    list.reserve(apps.size());
+    for (const auto& a: apps) {
         list.append(QVariant::fromValue<NotifyingApplication>(a));
     }
     config()->setList(QStringLiteral("applications"), list);
