@@ -1,21 +1,7 @@
 /**
- * Copyright 2014 Samoilenko Yuri<kinnalru@gmail.com>
+ * SPDX-FileCopyrightText: 2014 Samoilenko Yuri <kinnalru@gmail.com>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of
- * the License or (at your option) version 3 or any later version
- * accepted by the membership of KDE e.V. (or its successor approved
- * by the membership of KDE e.V.), which shall act as a proxy
- * defined in Section 14 of version 3 of the license.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
 
 #include "mounter.h"
@@ -28,7 +14,7 @@
 
 #include "mountloop.h"
 #include "config-sftp.h"
-#include "sftp_debug.h"
+#include "plugin_sftp_debug.h"
 #include "kdeconnectconfig.h"
 
 Mounter::Mounter(SftpPlugin* sftp)
@@ -69,7 +55,7 @@ bool Mounter::wait()
     qCDebug(KDECONNECT_PLUGIN_SFTP) << "Starting loop to wait for mount";
 
     MountLoop loop;
-    connect(this, &Mounter::mounted, &loop, &MountLoop::successed);
+    connect(this, &Mounter::mounted, &loop, &MountLoop::succeeded);
     connect(this, &Mounter::failed, &loop, &MountLoop::failed);
     return loop.exec();
 }
@@ -125,6 +111,10 @@ void Mounter::onPackageReceived(const NetworkPacket& np)
         return;
     }
     QString ip = addr.toString();
+    if (addr.protocol() == QAbstractSocket::IPv6Protocol) {
+        ip.prepend(QLatin1Char('['));
+        ip.append(QLatin1Char(']'));
+    }
 
     const QStringList arguments = QStringList()
         << QStringLiteral("%1@%2:%3").arg(
@@ -139,7 +129,7 @@ void Mounter::onPackageReceived(const NetworkPacket& np)
         << QStringLiteral("-o") << QStringLiteral("IdentityFile=") + KdeConnectConfig::instance().privateKeyPath()
         << QStringLiteral("-o") << QStringLiteral("StrictHostKeyChecking=no") //Do not ask for confirmation because it is not a known host
         << QStringLiteral("-o") << QStringLiteral("UserKnownHostsFile=/dev/null") //Prevent storing as a known host
-        << QStringLiteral("-o") << QStringLiteral("HostKeyAlgorithms=+ssh-dss") //https://bugs.kde.org/show_bug.cgi?id=351725
+        << QStringLiteral("-o") << QStringLiteral("HostKeyAlgorithms=+ssh-dss\\,ssh-rsa") //https://bugs.kde.org/show_bug.cgi?id=351725
         << QStringLiteral("-o") << QStringLiteral("uid=") + QString::number(getuid())
         << QStringLiteral("-o") << QStringLiteral("gid=") + QString::number(getgid())
         << QStringLiteral("-o") << QStringLiteral("reconnect")
