@@ -18,15 +18,79 @@
 #include <core/device.h>
 
 #include "plugin_systemvolume_debug.h"
+#include "PolicyConfig.h"
 
 K_PLUGIN_CLASS_WITH_JSON(SystemvolumePlugin, "kdeconnect_systemvolume.json")
 
 // Private classes of SystemvolumePlugin
 
+class SystemvolumePlugin::CAudioEndpointVolumeCallback : public IAudioEndpointVolumeCallback
+{
+    LONG _cRef;
+
+public:
+    CAudioEndpointVolumeCallback(SystemvolumePlugin &x, QString sinkName) : enclosing(x), name(std::move(sinkName)), _cRef(1) {}
+    ~CAudioEndpointVolumeCallback(){};
+
+    // IUnknown methods -- AddRef, Release, and QueryInterface
+
+    ULONG STDMETHODCALLTYPE AddRef() override
+    {
+        return InterlockedIncrement(&_cRef);
+    }
+
+    ULONG STDMETHODCALLTYPE Release() override
+    {
+        ULONG ulRef = InterlockedDecrement(&_cRef);
+        if (ulRef == 0)
+        {
+            delete this;
+        }
+        return ulRef;
+    }
+
+    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, VOID **ppvInterface) override
+    {
+        if (IID_IUnknown == riid)
+        {
+            AddRef();
+            *ppvInterface = (IUnknown *)this;
+        }
+        else if (__uuidof(IMMNotificationClient) == riid)
+        {
+            AddRef();
+            *ppvInterface = (IMMNotificationClient *)this;
+        }
+        else
+        {
+            *ppvInterface = NULL;
+            return E_NOINTERFACE;
+        }
+        return S_OK;
+    }
+
+    // Callback method for endpoint-volume-change notifications.
+
+    HRESULT STDMETHODCALLTYPE OnNotify(PAUDIO_VOLUME_NOTIFICATION_DATA pNotify) override
+    {
+        NetworkPacket np(PACKET_TYPE_SYSTEMVOLUME);
+        np.set<int>(QStringLiteral("volume"), (int)(pNotify->fMasterVolume * 100));
+        np.set<bool>(QStringLiteral("muted"), pNotify->bMuted);
+        np.set<QString>(QStringLiteral("name"), name);
+        enclosing.sendPacket(np);
+
+        return S_OK;
+    }
+
+private:
+    SystemvolumePlugin &enclosing;
+    QString name;
+};
+
 class SystemvolumePlugin::CMMNotificationClient : public IMMNotificationClient
 {
 
-  public:
+public:
     CMMNotificationClient(SystemvolumePlugin &x) : enclosing(x), _cRef(1){};
 
     ~CMMNotificationClient(){};
@@ -85,14 +149,59 @@ class SystemvolumePlugin::CMMNotificationClient : public IMMNotificationClient
         return S_OK;
     }
 
+    struct RemovedDeviceThreadData {
+
+        QString qDeviceId;
+        SystemvolumePlugin* plugin;
+
+    };
+
+    static DWORD WINAPI releaseRemovedDevice(_In_ LPVOID lpParameter) {
+
+        auto* data = static_cast<RemovedDeviceThreadData*>(lpParameter);
+
+        if (!data->plugin->sinkList.empty())
+        {
+            auto idToNameIterator = data->plugin->idToNameMap.find(data->qDeviceId);
+            if (idToNameIterator == data->plugin->idToNameMap.end()) return 0;
+
+            QString& sinkName = idToNameIterator.value();
+
+            auto sinkListIterator = data->plugin->sinkList.find(sinkName);
+            if (sinkListIterator == data->plugin->sinkList.end()) return 0;
+
+            auto& sink = sinkListIterator.value();
+
+            sink.first->UnregisterControlChangeNotify(sink.second);
+            sink.first->Release();
+            sink.second->Release();
+
+            data->plugin->idToNameMap.remove(data->qDeviceId);
+            data->plugin->sinkList.remove(sinkName);
+        }
+
+        data->plugin->sendSinkList();
+
+        return 0;
+    }
+
     HRESULT STDMETHODCALLTYPE OnDeviceRemoved(LPCWSTR pwstrDeviceId) override
     {
-        enclosing.sendSinkList();
+        static RemovedDeviceThreadData data {};
+        data.qDeviceId = QString::fromWCharArray(pwstrDeviceId);
+        data.plugin = &enclosing;
+
+        DWORD threadId;
+        HANDLE threadHandle = CreateThread(NULL, 0, releaseRemovedDevice, &data, 0, &threadId);
+        CloseHandle(threadHandle);
+
         return S_OK;
     }
 
     HRESULT STDMETHODCALLTYPE OnDeviceStateChanged(LPCWSTR pwstrDeviceId, DWORD dwNewState) override
     {
+        if (dwNewState == DEVICE_STATE_UNPLUGGED) return OnDeviceRemoved(pwstrDeviceId);
+
         enclosing.sendSinkList();
         return S_OK;
     }
@@ -103,74 +212,11 @@ class SystemvolumePlugin::CMMNotificationClient : public IMMNotificationClient
         return S_OK;
     }
 
-  private:
+private:
     LONG _cRef;
     SystemvolumePlugin &enclosing;
 };
 
-class SystemvolumePlugin::CAudioEndpointVolumeCallback : public IAudioEndpointVolumeCallback
-{
-    LONG _cRef;
-
-  public:
-    CAudioEndpointVolumeCallback(SystemvolumePlugin &x, QString sinkName) : enclosing(x), name(sinkName), _cRef(1) {}
-    ~CAudioEndpointVolumeCallback(){};
-
-    // IUnknown methods -- AddRef, Release, and QueryInterface
-
-    ULONG STDMETHODCALLTYPE AddRef() override
-    {
-        return InterlockedIncrement(&_cRef);
-    }
-
-    ULONG STDMETHODCALLTYPE Release() override
-    {
-        ULONG ulRef = InterlockedDecrement(&_cRef);
-        if (ulRef == 0)
-        {
-            delete this;
-        }
-        return ulRef;
-    }
-
-    HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, VOID **ppvInterface) override
-    {
-        if (IID_IUnknown == riid)
-        {
-            AddRef();
-            *ppvInterface = (IUnknown *)this;
-        }
-        else if (__uuidof(IMMNotificationClient) == riid)
-        {
-            AddRef();
-            *ppvInterface = (IMMNotificationClient *)this;
-        }
-        else
-        {
-            *ppvInterface = NULL;
-            return E_NOINTERFACE;
-        }
-        return S_OK;
-    }
-
-    // Callback method for endpoint-volume-change notifications.
-
-    HRESULT STDMETHODCALLTYPE OnNotify(PAUDIO_VOLUME_NOTIFICATION_DATA pNotify) override
-    {
-        NetworkPacket np(PACKET_TYPE_SYSTEMVOLUME);
-        np.set<int>(QStringLiteral("volume"), (int)(pNotify->fMasterVolume * 100));
-        np.set<bool>(QStringLiteral("muted"), pNotify->bMuted);
-        np.set<QString>(QStringLiteral("name"), name);
-        enclosing.sendPacket(np);
-
-        return S_OK;
-    }
-
-  private:
-    SystemvolumePlugin &enclosing;
-    QString name;
-};
-
 SystemvolumePlugin::SystemvolumePlugin(QObject *parent, const QVariantList &args)
     : KdeConnectPlugin(parent, args),
       sinkList()
@@ -204,19 +250,8 @@ bool SystemvolumePlugin::sendSinkList()
     QJsonDocument document;
     QJsonArray array;
 
-    HRESULT hr;
-    if (!sinkList.empty())
-    {
-        for (auto const &sink : sinkList)
-        {
-            sink.first->UnregisterControlChangeNotify(sink.second);
-            sink.first->Release();
-            sink.second->Release();
-        }
-        sinkList.clear();
-    }
     IMMDeviceCollection *devices = nullptr;
-    hr = deviceEnumerator->EnumAudioEndpoints(eRender, DEVICE_STATE_ACTIVE, &devices);
+    HRESULT hr = deviceEnumerator->EnumAudioEndpoints(eRender, DEVICE_STATE_ACTIVE, &devices);
 
     if (hr != S_OK)
     {
@@ -226,6 +261,13 @@ bool SystemvolumePlugin::sendSinkList()
     }
     unsigned int deviceCount;
     devices->GetCount(&deviceCount);
+
+    IMMDevice *defaultDevice = nullptr;
+    deviceEnumerator->GetDefaultAudioEndpoint(eRender, eMultimedia, &defaultDevice);
+
+    LPWSTR defaultId = NULL;
+    defaultDevice->GetId(&defaultId);
+
     for (unsigned int i = 0; i < deviceCount; i++)
     {
         IMMDevice *device = nullptr;
@@ -234,6 +276,7 @@ bool SystemvolumePlugin::sendSinkList()
         PROPVARIANT deviceProperty;
         QString name;
         QString desc;
+        LPWSTR deviceId;
         float volume;
         BOOL muted;
 
@@ -252,6 +295,8 @@ bool SystemvolumePlugin::sendSinkList()
         deviceProperties->GetValue(PKEY_Device_DeviceDesc, &deviceProperty);
         desc = QString::fromWCharArray(deviceProperty.pwszVal);
         //PropVariantClear(&deviceProperty);
+#else
+        desc = name;
 #endif
 
         QJsonObject sinkObject;
@@ -267,22 +312,31 @@ bool SystemvolumePlugin::sendSinkList()
             device->Release();
             continue;
         }
+
+        device->GetId(&deviceId);
         endpoint->GetMasterVolumeLevelScalar(&volume);
         endpoint->GetMute(&muted);
 
         sinkObject.insert(QStringLiteral("muted"), (bool)muted);
         sinkObject.insert(QStringLiteral("volume"), (qint64)(volume * 100));
         sinkObject.insert(QStringLiteral("maxVolume"), (qint64)100);
+        sinkObject.insert(QStringLiteral("enabled"), lstrcmpW(deviceId, defaultId) == 0);
 
         // Register Callback
-        callback = new CAudioEndpointVolumeCallback(*this, name);
-        sinkList[name] = qMakePair(endpoint, callback);
-        endpoint->RegisterControlChangeNotify(callback);
+        if (!sinkList.contains(name)) {
+            callback = new CAudioEndpointVolumeCallback(*this, name);
+            endpoint->RegisterControlChangeNotify(callback);
+            sinkList[name] = qMakePair(endpoint, callback);
+        }
+
+        QString qDeviceId = QString::fromWCharArray(deviceId);
+        idToNameMap[qDeviceId] = name;
 
         device->Release();
         array.append(sinkObject);
     }
     devices->Release();
+    defaultDevice->Release();
 
     document.setArray(array);
 
@@ -302,6 +356,50 @@ void SystemvolumePlugin::connected()
     sendSinkList();
 }
 
+static HRESULT setDefaultAudioPlaybackDevice(PCWSTR deviceId) {
+    if (deviceId == nullptr) return ERROR_BAD_UNIT;
+
+    IPolicyConfigVista *pPolicyConfig;
+    HRESULT hr = CoCreateInstance(
+            __uuidof(CPolicyConfigVistaClient),
+            NULL,
+            CLSCTX_ALL,
+            __uuidof(IPolicyConfigVista),
+            (LPVOID *)&pPolicyConfig
+    );
+
+    if (SUCCEEDED(hr))
+    {
+        hr = pPolicyConfig->SetDefaultEndpoint(deviceId, eMultimedia);
+        pPolicyConfig->Release();
+    }
+
+    return hr;
+}
+
+HRESULT SystemvolumePlugin::setDefaultAudioPlaybackDevice(QString& name, bool enabled)
+{
+    if (!enabled) return S_OK;
+
+    PWSTR deviceId = nullptr;
+    for (auto& entry : idToNameMap.toStdMap()) {
+        if (entry.second == name) {
+
+            deviceId = new WCHAR[entry.first.length()];
+            wcscpy(deviceId, entry.first.toStdWString().data());
+            break;
+        }
+    }
+
+    if (deviceId == nullptr) return ERROR_BAD_UNIT;
+
+    HRESULT hr = ::setDefaultAudioPlaybackDevice(deviceId);
+
+    delete[] deviceId;
+
+    return hr;
+}
+
 bool SystemvolumePlugin::receivePacket(const NetworkPacket &np)
 {
     if (!valid)
@@ -317,13 +415,54 @@ bool SystemvolumePlugin::receivePacket(const NetworkPacket &np)
 
         if (sinkList.contains(name))
         {
+            // unregister ControlChangeNotify before doing any changes to a sink
+            HRESULT unregisterSuccess = E_POINTER;
+            auto sinkListIterator = this->sinkList.find(name);
+            auto &sink = sinkListIterator.value();
+            if (!(sinkListIterator == this->sinkList.end())) {
+                unregisterSuccess = sink.first->UnregisterControlChangeNotify(sink.second);
+            }
+
             if (np.has(QStringLiteral("volume")))
             {
-                sinkList[name].first->SetMasterVolumeLevelScalar((float)np.get<int>(QStringLiteral("volume")) / 100, NULL);
+                float currentVolume;
+                sink.first->GetMasterVolumeLevelScalar(&currentVolume);
+                float requestedVolume = (float)np.get<int>(QStringLiteral("volume"), 100) / 100;
+                if (currentVolume != requestedVolume) {
+                    sinkList[name].first->SetMasterVolumeLevelScalar(requestedVolume, NULL);
+                }
             }
+
             if (np.has(QStringLiteral("muted")))
             {
-                sinkList[name].first->SetMute(np.get<bool>(QStringLiteral("muted")), NULL);
+                BOOL currentMuteStatus;
+                sink.first->GetMute(&currentMuteStatus);
+                BOOL requestedMuteStatus = np.get<bool>(QStringLiteral("muted"), false);
+                if(currentMuteStatus != requestedMuteStatus) {
+                    sinkList[name].first->SetMute(requestedMuteStatus, NULL);
+                }
+            }
+
+            if (np.has(QStringLiteral("enabled")))
+            {
+                // get the current default device ID
+                IMMDevice *defaultDevice = nullptr;
+                deviceEnumerator->GetDefaultAudioEndpoint(eRender, eMultimedia, &defaultDevice);
+                LPWSTR defaultId = NULL;
+                defaultDevice->GetId(&defaultId);
+                defaultDevice->Release();
+                // get current sink's device ID
+                QString qDefaultId = QString::fromWCharArray(defaultId);
+                QString currentDeviceId = idToNameMap.key(name);
+
+                if ((bool)qDefaultId.compare(currentDeviceId)) {
+                    setDefaultAudioPlaybackDevice(name, np.get<bool>(QStringLiteral("enabled")));
+                }
+            }
+
+            // re-register ControlChangeNotify in case we unregistered it
+            if (unregisterSuccess == S_OK) {
+                sink.first->RegisterControlChangeNotify(sink.second);
             }
         }
     }
