@@ -15,15 +15,19 @@
 #include <QQmlApplicationEngine>
 #include <QCommandLineParser>
 #include <QQmlContext>
+#include <QQuickStyle>
 #include <KAboutData>
 #include <KLocalizedString>
 #include <KLocalizedContext>
 #include <KDBusService>
+#include <KColorSchemeManager>
 
 #include "smshelper.h"
 
 int main(int argc, char *argv[])
 {
+    QIcon::setFallbackThemeName(QStringLiteral("breeze"));
+
     QApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
     QApplication app(argc, argv);
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("kdeconnect")));
@@ -38,6 +42,16 @@ int main(int argc, char *argv[])
     aboutData.addAuthor(i18n("Nicolas Fella"), {}, QStringLiteral("nicolas.fella@gmx.de"));
     KAboutData::setApplicationData(aboutData);
 
+#ifdef Q_OS_WIN
+    KColorSchemeManager manager;
+    QApplication::setStyle(QStringLiteral("breeze"));
+#endif
+
+    // Default to org.kde.desktop style unless the user forces another style
+    if (qEnvironmentVariableIsEmpty("QT_QUICK_CONTROLS_STYLE")) {
+        QQuickStyle::setStyle(QStringLiteral("org.kde.desktop"));
+    }
+
     QString initialMessage, deviceid;
 
     {
